#!/bin/bash

# -------------------------------------------------------------------------- #
# Copyright 2002-2009, Distributed Systems Architecture Group, Universidad   #
# Complutense de Madrid (dsa-research.org)                                   #
#                                                                            #
# Licensed under the Apache License, Version 2.0 (the "License"); you may    #
# not use this file except in compliance with the License. You may obtain    #
# a copy of the License at                                                   #
#                                                                            #
# http://www.apache.org/licenses/LICENSE-2.0                                 #
#                                                                            #
# Unless required by applicable law or agreed to in writing, software        #
# distributed under the License is distributed on an "AS IS" BASIS,          #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   #
# See the License for the specific language governing permissions and        #
# limitations under the License.                                             #
#--------------------------------------------------------------------------- #

SRC=$1
DST=$2

if [ $# -gt 2 ]; then
    SIZE=$3
fi

if [ -z "${ONE_LOCATION}" ]; then
    TMCOMMON=/usr/lib/one/mads/tm_common.sh
else
    TMCOMMON=$ONE_LOCATION/lib/mads/tm_common.sh
fi

. $TMCOMMON

SRC_PATH=`arg_path $SRC`
DST_PATH=`arg_path $DST`

SRC_HOST=`arg_host $SRC`
DST_HOST=`arg_host $DST`

if [ $# -gt 2 ]; then
    log "$1 $2 $3"
else
    log "$1 $2"
fi

log "DST: $DST_PATH"

DST_DIR=`dirname $DST_PATH`

log "Creating directory $DST_DIR"
exec_and_log "ssh $DST_HOST mkdir -p $DST_DIR"

case $SRC_PATH in
#http://*)
#    log "Downloading $SRC"
#    exec_and_log "ssh $DST_HOST wget -O $DST_PATH $SRC_PATH"
#    ;;
#
/dev/*)
    log "Cloning LVM $SRC_PATH"
    # Default to 1G clone size
    if [ $# -lt 3 ]; then
        SIZE="1G"
    fi
    # Combine VID number and disk number into LV name and create it
    LV_NAME=`/bin/echo ${DST_PATH}|/bin/awk -F "/" '{print "vid-" $(NF-2) "." $(NF)}'`
    VG_PATH=`/usr/bin/dirname ${SRC_PATH}`
    COW_NAME="${LV_NAME}-cow"
    COW_PATH="${VG_PATH}/${COW_NAME}"
    exec_and_log "ssh $DST_HOST sudo /usr/sbin/lvcreate -L$SIZE -n ${COW_NAME} ${VG_PATH}"

    # Disable COW on all nodes and enable it exclusively on the destination host
    exec_and_log "ssh $DST_HOST sudo /usr/sbin/lvchange -an ${COW_PATH}"
    exec_and_log "ssh $DST_HOST sudo /usr/sbin/lvchange -aey ${COW_PATH}"

    # Get device numbers and size needed for dmsetup
    SRC_DEV=`ssh $DST_HOST sudo /usr/sbin/lvs --noheadings --aligned --separator ":" -o lv_kernel_major,lv_kernel_minor $SRC_PATH | /bin/sed 's/ //g'`
    SRC_SIZE=`ssh $DST_HOST sudo /sbin/blockdev --getsize $SRC_PATH`
    COW_DEV=`ssh $DST_HOST sudo /usr/sbin/lvs --noheadings --aligned --separator ":" -o lv_kernel_major,lv_kernel_minor ${COW_PATH} | sed 's/ //g'`

    # Construct snapshot name from device mapper path
    MAP_PATH=`ssh $DST_HOST /bin/ls -l ${COW_PATH}|/bin/awk -F "->" '{print $(NF)}'`
    MAP_NAME=`/bin/basename ${MAP_PATH}`
    MAP_NAME=${MAP_NAME%--cow}

    # Create snapshot interface
    exec_and_log "ssh $DST_HOST sudo /sbin/dmsetup create ${MAP_NAME} --table \"0 $SRC_SIZE snapshot $SRC_DEV $COW_DEV P 8\""
    exec_and_log "ssh $DST_HOST /bin/ln -s /dev/mapper/${MAP_NAME} ${VG_PATH}/${LV_NAME}"
    ;;

*)
    log "Cloning $SRC"
    exec_and_log "ssh $DST_HOST cp $SRC_PATH $DST_PATH"
    exec_and_log "ssh $DST_HOST chmod a+w $DST_PATH"
    ;;
esac
